<?php

class ScreenInfo extends CModel {
	
	var $resolution;
    var $refresh;
    var $aspect;
    var $monitors;
    var $powersave;
    
    
    public function attributeNames() {
        return array("resolution", "refresh", "aspect", "monitors" );
    }
    public function safeAttributes() {
        return array();
    }
    public function load() {
        $dir = self::sysfs_vout_device();
        
        if ( file_exists($dir . "vid_mode") ) {
            $vidmod=file_get_contents( $dir . "vid_mode");
            $sets=explode ("=",$vidmod);
            if (count($sets)==2){
                $vals=explode (" ",trim($sets[1]));
                if (count($vals)>=11){
                    if ($vals[11]=="0:0") $vals[11]="[auto]";
                    if ($vals[11]=="x:x") $vals[11]="[unknown]";
                    $this->resolution = htmlspecialchars($vals[0])."x".htmlspecialchars($vals[1]);
                    $this->refresh = htmlspecialchars($vals[2]);
                    $this->aspect = htmlspecialchars($vals[11]);
                }
            }
        } else {
            $this->resolution = "unknown";
            $this->refresh = "unknown";
            $this->aspect = "unknown";
        }
        
        $dpm_state=file_get_contents($dir . "dpm_state");
        $this->powersave = $dpm_state[0]=="0"?"on":"off";
        
        
        $this->monitors = array();
        $content=scandir( $dir );
        foreach($content as $path){
            if (strncmp($path,"out",3)==0 && file_exists("$dir$path/name") ){
                $result = array();
                $name=file_get_contents("$dir$path/name");
                $result['name'] = htmlspecialchars($name);
                $result['mon_present'] = htmlspecialchars(file_get_contents("$dir$path/mon_present"));
                
                if ($result['mon_present']==1) {
                    $data=array("mon_powered","mon_manufacturer","mon_prod_id", "mon_name","mon_serial_no", "mon_width","mon_height","mon_underscan","mon_audio","mon_pict_ar" );
                    foreach ($data as $d){
                        if ( file_exists( $dir.$path."/$d" ) )
                            $result[$d]=file_get_contents($dir.$path."/$d");
                        else
                            $result[$d]="";
                    }
                    if ($result['mon_pict_ar']=="" || $result['mon_pict_ar']=="x:x") $result['mon_pict_ar']="[unknown]";
                }
                $this->monitors[] = $result;
            }
        }
    }
    function getAspectRatio() {
        if ( count( $this->monitors ) ==0 )
            return "16:9";    
        $result = $this->monitors[0];
		if ( $result['mon_width']!="" && $result['mon_height']!="" ) {
			$ar = floatval($result['mon_width'])/floatval($result['mon_height']);
			if ( $ar <1.4 ) 
				return "4:3";
			else if ( $ar<1.7 )
				return "16:10";
			else
				return "16:9";
		} else if ( $result['mon_pict_ar']=="4:3" || $result['mon_pict_ar']=="16:10" || $result['mon_pict_ar']=="16:9")
			return $result['mon_pict_ar'];
		return "16:9";
	}
    
    # Returns the location of the video out sysfs, with a trailing slash
    static function sysfs_vout_device() {
        if (file_exists('/sys/class/x-display/disp0/device')) {
            return '/sys/class/x-display/disp0/device/';
        } else {
            return '/sys/class/graphics/fb0/device/';
        }
    }
	
}